#!/usr/bin/env python

# The Python version of qwt-*/examples/simple_plot/simple.cpp

import glob, os, sys

# The next statements assure importation of qwt from the
# local source tree. This is useful for debugging and testing.
globs = glob.glob('../build/lib.*/qwt')
if globs:                               # found PyQwt modules made by distutils
    sys.path.insert(0, os.path.split(globs[0])[0])

from qt import *
from qwt import *
from Numeric import *

class SimplePlot(QwtPlot):

    def __init__(self, *args):
        apply(QwtPlot.__init__, (self,) + args)
	# make a QwtPlot widget
	self.setTitle('A Simple PyQwtPlot Demonstration')
        self.setAutoLegend(1)
        self.setLegendPos(Qwt.Right)
	# set axis titles
	self.setAxisTitle(QwtPlot.xBottom, 'x -->')
	self.setAxisTitle(QwtPlot.yLeft, 'y -->')
	# insert a few curves
	cSin = self.insertCurve('y = sin(x)')
	cCos = self.insertCurve('y = cos(x)')
	# set curve styles
	self.setCurvePen(cSin, QPen(Qt.red))
	self.setCurvePen(cCos, QPen(Qt.blue))
	# calculate 3 NumPy arrays
        x = arrayrange(0.0, 10.0, 0.1)
        y = sin(x)
        z = cos(x)
	# copy the data
	self.setCurveData(cSin, x, y)
	self.setCurveData(cCos, x, z)
	# insert a horizontal marker at y = 0
	mY = self.insertLineMarker('y = 0', QwtPlot.yLeft)
	self.setMarkerYPos(mY, 0.0)
	# insert a vertical marker at x = 2 pi
	mX = self.insertLineMarker('x = 2 pi', QwtPlot.xBottom)
	self.setMarkerXPos(mX, 2*pi)
        # replot
        self.replot()
	

# admire	
app = QApplication(sys.argv)
demo = SimplePlot()
app.setMainWidget(demo)
demo.resize(500, 300)
demo.show()
app.exec_loop()




