#!/usr/bin/env python

import os

# EDIT the Windows or Linux/Unix section to reflect your system:
# qt_dir -- must contain your Qt distribution (headers, binaries, libraries)
# sip_program -- must be the full path to the sip program
# sip_spec_dirs -- must contain the files 'qtmod.sip' & friends

if os.name == 'nt':
    # Windows-95, -98, ...
    # My PyQwt build procedure is (see HTML documentation):
    # (1) build and install sip from source (see sip-3.3.2\README)
    # (2) build and install PyQt from source (see PyQt-3.3.3\README)
    # (3) build PyQwt from source
    qt_dir = 'C:\qt'
    sip_program = 'sip.exe' # by default sip is installed in C:\Python2.2.
    sip_spec_dirs = ['C:\MyStuff\PyQt-3.3.2\sip']
else:
    # Linux/Unix
    qt_dir = '/usr/lib/qt3'
    sip_program = '/usr/bin/sip'
    sip_spec_dirs = ['/usr/share/sip/qt']

# IF YOU NEED TO EDIT BELOW THIS LINE -- IT IS A BUG

qwtdir = 'qwt-0.4.1'

import sys

# os.popen should work on Unix AND Windows for python >= 2.0
if not hasattr(sys, 'version_info') or sys.version_info < (2,0,0, 'alpha', 0):
    raise SystemExit, 'Python-2.0 or later required to build PyQwt.'

file = open('setup.cfg', 'w')
file.write('[build_ext]\n')
file.write('qt_dir=%s\n' % qt_dir)
file.write('sip_program=%s\n' % sip_program)
file.close()
    
from distutils.core import setup
from distutils.file_util import copy_file
from distutils.sysconfig import get_python_inc
from glob import glob
from os import mkdir
from os.path import exists, join
from pyqt_support import PyQtExtension, build_ext_pyqt, install_data_pyqt

if not exists('qwt'):
    mkdir('qwt')

data_files = []
if os.name == 'nt':
    for filename in ['BodeDemo.py',
                     'CurveDemo1.py',
                     'CurveDemo2.py',
                     'DataDemo.py',
                     'ListArrayDemo.py',
                     'MinPackDemo.py',
                     'RadioDemo.py',
                     'SimpleDemo.py',
                     'SliderDemo.py']:
        copy_file('examples/%s' % filename, 'examples/%sw' % filename)
    data_files.append(('qwt/examples',
                       ['examples/README'] +
                       glob('examples/*.pyw')))
else:
    data_files.append(('qwt/examples',
                       ['examples/README'] +
                       glob('examples/*.py')))

pyuic_files =  ['LegendBaseDialog.py',
                'LegendBaseDialog.ui',
                'LegendDialog.py',
                'QwtCounter.cw',
                'QwtCounter.xpm',
                'QwtPlot.cw',
                'QwtPlot.xpm',
                'README',
                'make.py']

from qt import QT_VERSION
if QT_VERSION<300:
    data_files.append(('qwt/pyuic',
                       map(lambda x: os.path.join('pyuic', x), pyuic_files)))
else:
    data_files.append(('qwt/pyuic',
                       map(lambda x: os.path.join('pyuic3', x), pyuic_files)))

data_files.append(('qwt/doc', glob('doc/PyQwt/*.html')))

source_files = glob(join(qwtdir, 'src', '*.cpp'))
source_files.append(join('numeric', 'qwt_numeric.cpp'))

pyQwt = PyQtExtension('PyQwt.qwt',
                      source_files,
                      sip_spec_dirs = sip_spec_dirs,
                      moc_headers = ['%s/include/qwt_counter.h' % qwtdir,
                                     '%s/include/qwt_legend.h' % qwtdir,
                                     '%s/include/qwt_plot.h' % qwtdir,
                                     '%s/include/qwt_plot_canvas.h' % qwtdir,
                                     '%s/include/qwt_sldbase.h' % qwtdir,
                                     '%s/include/qwt_thermo.h' % qwtdir,
                                     '%s/include/qwt_wheel.h' % qwtdir ],
                      include_dirs = ['numeric',
                                      '%s/include' % qwtdir])
#                      libraries = libraries)

long_description = """
PyQwt = FAST and EASY data plotting for Python, Numeric and Qt!

PyQwt is a set of Python bindings for the Qwt C++ class library.
The Qwt library extends the Qt framework with widgets for
Scientific and Engineering applications.  It contains QwtPlot,
a 2d plotting widget, and widgets for data input/output such as
and QwtCounter, QwtKnob, QwtThermo and QwtWheel. 

PyQwt requires and extends PyQt, a set of Python bindings for Qt.

PyQwt requires Numeric.  Numeric extends the Python language
with new data types that make Python an ideal language for
numerical computing and experimentation (maybe less efficient
than MatLab, but more expressive).  
"""

setup(name              = "PyQwt",
      version           = 'sip332_041',
      licence           = "LGPL",
      url               = "http://gerard.vermeulen.free.fr",
      author            = "Gerard Vermeulen",
      author_email      = "gvermeul@grenoble.cnrs.fr",
      description       = "Python bindings for the Qwt library",
      long_description  = long_description,
      cmdclass          = {'build_ext': build_ext_pyqt,
                           'install_data': install_data_pyqt},
      packages          = ['qwt'],
      package_dir       = {'qwt': 'lib-qwt'},
      ext_modules       = [pyQwt],
      data_files        = data_files)

# Local Variables: ***
# compile-command: "python setup.py build" ***
# End: ***
